<?php

namespace WBundler {

	use stdClass;

	/**
	 * Main WBundler class.
	 *
	 * @since 1.0.0
	 */
	final class WBundler {

		/**
		 * Bundler Instance
		 *
		 * @since 1.0.0
		 *
		 */
		private static $instance;

		/**
		 * Plugin version for enqueueing, etc.
		 * The value is got from WBDL_VERSION constant.
		 *
		 * @since 1.0.0
		 *
		 * @var string
		 */
		public $version = '';

		/**
		 * Plugin settings.
		 *
		 * @since 1.0.0
		 *
		 * @var Settings
		 */
		private $settings;

		/**
		 * Plugin License.
		 *
		 * @since 1.0.0
		 *
		 * @var string
		 */
		private $license;

		/**
		 * Pro returns true, free (Lite) returns false.
		 *
		 * @since 1.0.0
		 *
		 * @var bool
		 */
		private $pro = false;


		/**
		 * Main WBundler Instance.
		 *
		 * Only one instance of Bundler exists in memory at any one time.
		 * Also prevent the need to define globals all over the place.
		 *
		 * @since 1.0.0
		 *
		 * @return WBundler
		 */
		public static function instance() {

			if (
				self::$instance === null ||
				! self::$instance instanceof self
			) {

				self::$instance = new self();

				self::$instance->includes();
				self::$instance->hooks();
				self::$instance->constants();

				add_action( 'init', [ self::$instance, 'load_textdomain' ], 10 );
				add_action( 'plugins_loaded', [ self::$instance, 'objects' ], 10 );
			}

			return self::$instance;
		}

		/**
		 * Register hooks.
		 *
		 * @since 1.0.0
		 */
		public function hooks() {

			// Enqueue scripts
			add_action( 'admin_enqueue_scripts', [ $this, 'wbdl_enqueue_admin_style' ] );
			add_action( 'wp_enqueue_scripts', [ $this, 'wbdl_enqueue_script' ] );

			// Add the settings page link to plugins page
			add_filter('plugin_action_links_' . WBDL_PLUGIN_BASENAME, function ( $links ) {
			    $links[] = '<a href="' .
			        admin_url( 'admin.php?page=woobundles#general-settings' ) .
			        '">' . __('Settings') . '</a>';
			    return $links;
			});

			// Add the docs page link to plugins page
			add_filter('plugin_action_links_' . WBDL_PLUGIN_BASENAME, function ( $links ) {
			    $links[] = '<a target="_blank" href="https://woobundles.com/docs">' . __('Docs') . '</a>';
			    return $links;
			});

			// Add the Go Pro link to plugins page
			add_filter('plugin_action_links_'.plugin_basename(__FILE__), 'wbdl_go_pro');

			function wbdl_go_pro( $links ) {
			    $links[] = '<a target="_blank" href="https://woobundles.com/pricing" style="color:#93003c;font-weight:700;">' . __('Go Pro') . '</a>';
			    return $links;
			}
			
			// Register activation hook for DB required tables creation
			register_activation_hook( WBDL_PLUGIN_FILE, function (){
	    		\DBTables::createDbTables();
			});

		}

		/**
		 * Enqueue Scripts.
		 *
		 * @since 1.0.0
		 */
		public function wbdl_enqueue_script() {
			wp_register_script( 'custom_jquery', WBDL_PLUGIN_URL.'assets/js/custom-jquery.js', array('jquery'), '2.4.1' );
			wp_localize_script( 'custom_jquery', 'wbdl_ajax', array( 'ajaxurl' => admin_url( 'admin-ajax.php' )) );
			wp_register_style( 'custom_style', WBDL_PLUGIN_URL.'assets/css/custom-style.css', false, WBDL_VERSION, 'all');
			wp_enqueue_script('custom_jquery');
			wp_enqueue_style( 'custom_style' );
		}

		/**
		 * Enqueue Admin CSS styles.
		 *
		 * @since 1.0.0
		 */
		public function wbdl_enqueue_admin_style() {

		    wp_enqueue_style('admin-settings', WBDL_PLUGIN_URL.'assets/admin/css/admin-settings.css', false, WBDL_VERSION, 'all');
		    wp_enqueue_style('admin-bundle-form', WBDL_PLUGIN_URL.'assets/admin/css/admin-bundle-form.css', false, WBDL_VERSION, 'all');
		    wp_enqueue_style('admin-coloris-css', WBDL_PLUGIN_URL.'assets/admin/css/coloris.min.css', false, WBDL_VERSION, 'all');
			wp_enqueue_script( 'admin_jquery', WBDL_PLUGIN_URL.'assets/admin/js/admin-custom.js', array('jquery'), '2.5.1' );
		    wp_enqueue_script( 'admin_coloris_js', WBDL_PLUGIN_URL.'assets/admin/js/coloris.min.js');
		}

		/**
		 * Setup plugin constants.
		 * All the path/URL related constants are defined in main plugin file.
		 *
		 * @since 1.0.0
		 */
		private function constants() {

			$this->version = WBDL_VERSION;

			// Plugin Slug - Determine plugin type and set slug accordingly.
			
			if ( WBDL_PRO == true ) {
				$this->pro = true;
				define( 'WBDL_PLUGIN_SLUG', 'bundler-pro' );
			} else {
				$this->pro = false;
				define( 'WBDL_PLUGIN_SLUG', 'bundler-lite' );
			}
		}


		/**
		 * Load the plugin language files.
		 *
		 * @since 1.0.0
		 * 
		 */
		public function load_textdomain() {

			// If the user is logged in, unset the current text-domains before loading our text domain.
			// This feels hacky, but this way a user's set language in their profile will be used,
			// rather than the site-specific language.
			if ( is_user_logged_in() ) {
				unload_textdomain( 'woobundles' );
			}

			load_plugin_textdomain( 'woobundles', false, dirname( plugin_basename( WBDL_PLUGIN_FILE ) ) . '/assets/languages/' );
		}

		/**
		 * Include files.
		 *
		 * @since 1.0.0
		 */
		private function includes() {

			// Load classes
			require_once WBDL_PLUGIN_DIR . 'App/Models/DBTables.php';
			require_once WBDL_PLUGIN_DIR . 'App/Controllers/BundleController.php';
			require_once WBDL_PLUGIN_DIR . 'App/Controllers/SettingsController.php';

			//Load Ajax Add to Cart
		    require_once WBDL_PLUGIN_DIR . 'App/Snippets/ajax-add-to-cart.php';

			// Admin/Dashboard only includes, also in ajax.
			if ( is_admin() ) {
				// Load the plugin's settings section.
    			require_once WBDL_PLUGIN_DIR . 'App/Views/admin/admin-global-settings.php';

    			// Load the plugin's admin section.
    			require_once WBDL_PLUGIN_DIR . 'App/Views/admin/admin-bundle-section.php';
			}

		}


		/**
		 * Setup objects.
		 *
		 * @since 1.0.0
		 */
		public function objects() {

			// Global objects.
			$this->settingsController = new \SettingsController();
			$this->settings = $this->settingsController->getSettings();

			require_once WBDL_PLUGIN_DIR . 'App/Views/classic-view.php';
		    
			// Hook now that all of the WBundler stuff is loaded.
			do_action( 'wbdl_loaded' );
		}


		/**
		 * Get the list of all custom tables starting with `wbdl_*`.
		 *
		 * @since 1.0.0
		 *
		 * @return array List of table names.
		 */
		public function get_existing_custom_tables() {

			global $wpdb;

			$tables = $wpdb->get_results( "SHOW TABLES LIKE '" . $wpdb->prefix . "wbdl_%'", 'ARRAY_N' ); // phpcs:ignore

			return ! empty( $tables ) ? wp_list_pluck( $tables, 0 ) : [];
		}

		/**
		 * Whether the current instance of the plugin is a paid version, or free.
		 *
		 * @since 1.0.0
		 *
		 * @return bool
		 */
		public function is_pro() {

			/**
			 * Filters whether the current plugin version is pro.
			 *
			 * @since 1.0.0
			 *
			 * @param bool $pro Whether the current plugin version is pro.
			 */
			return (bool) $this->pro;
		}
	}
}

namespace {

	/**
	 * The function which returns the one WBundler instance.
	 *
	 * @since 1.0.0
	 *
	 * @return WBundler\WBundler
	 */
	function wbdl() {

		return WBundler\WBundler::instance();
	}

	/**
	 * Adding an alias for backward-compatibility with plugins
	 * that still use class_exists( 'WBundler' )
	 * instead of function_exists( 'wbdl' ), which is preferred.
	 *
	 * @since 1.0.0
	 */
	class_alias( 'WBundler\WBundler', 'WBundler' );
}
